<!--- 
	csw-scroll-spy-container.cfc
	
	This is the core rendering for the Scroll Spy widget.
	
	Version: [2018-01-17 10:45]
--->

<cfcomponent displayname="csw-scroll-spy-container" output="false" hint="Common Scroll Spy Functions" extends="_cs_widgets._common.csw-common">

	<cffunction name="loadResources" access="public" output="Yes" returntype="void">
		<cfscript>
			// load Resources
			Server.CommonSpot.udf.resources.loadResources("JQuery");
			Server.CommonSpot.udf.resources.loadResources("Bootstrap");
			Server.CommonSpot.udf.resources.loadResources("cswCommonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswButtonStyles");
			Server.CommonSpot.udf.resources.loadResources("cswTextStyles");
			Server.CommonSpot.udf.resources.loadResources("cswScrollSpyContainer");

			// If a custom stylesheet has been registered, go ahead and load it.

			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswCommonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswCommonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswButtonStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswButtonStyles_custom");
			if( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswTextStyles_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswTextStyles_custom");

			if ( StructKeyExists(Request.Site.CS_Resources.Resources_by_name,'cswScrollSpyContainer_custom') )
				Server.CommonSpot.udf.resources.loadResources("cswScrollSpyContainer_custom");
		</cfscript>
	</cffunction>


	
	<cffunction name="loadProperties" access="public" returntype="any" output="Yes">
		<cfargument name="elementInfo" type="struct" required="Yes">
		<cfargument name="rhCaller" type="struct" required="false" default="#StructNew()#">
		<cfargument name="showDataError" type="boolean" required="false" default="true">  
		
		<cfscript>
			var widgetMetadataFormName = "CSW Scroll Spy";
			var props = StructNew();
			var rhMetadata = arguments.ElementInfo.RenderHandlerMetaData;
			var widgetData = StructNew();
			var useScrollSpy = false;
			
			
			// Check for Props and Layout Binding
			if( NOT hasPropsLayoutBinding(ElementInfo=arguments.elementInfo,metadataFormName=widgetMetadataFormName,rhCaller=arguments.rhCaller) )
				return props;

			// 
			// Get Metadata (Core Scroll Spy metadata)
			// ----------------------------------------------
			widgetData = rhMetadata[widgetMetadataFormName];
			
			
			// Set props defaults
			props.themeClass = "";
			props.bodycontentClass="";
			
			// Theme
			if( StructKeyExists(widgetData,"Theme") )
				props.themeClass = TRIM(widgetData.Theme);
				
			/**** SCROLL SPY OPTIONS TAB *****/
			
			// Enable_Menu
			props.EnableMenu = getStandardField( widgetData=widgetData, field='Enable_Menu', defValue='false' );
			
			if ( IsBoolean(props.EnableMenu) AND props.EnableMenu )
				useScrollSpy = true;
			
			// Fixed_Menu_Top
			props.FixedMenuTop = getStandardField( widgetData=widgetData, field='Fixed_Menu_Top', defValue='90' );

			// Scroll_Offset
			props.ScrollOffset = getStandardField( widgetData=widgetData, field='Scroll_Offset', defValue='100' );
			
			// Affix_Offset
			props.AffixOffset = getStandardField( widgetData=widgetData, field='Affix_Offset', defValue='150' );
			
			/**** TEXT & LINK PROPERTIES TAB *****/
			
			// Title_Text
			props.TitleText = getStandardField( widgetData=widgetData, field='Title_Text', defTag='p');
			
			// Title_Style (struct)
			props.TitleStyle = getFieldSettings(widgetData=widgetData, field='Title_Style', defShow='1', defTag='H2');
			
			// Link_Style (struct)
			props.MenuLinksStyle = getFieldSettings(widgetData=widgetData, field='Link_Style', defShow='1', defAlign='cswTextAlign-left');
			
			/**** SCROLL SPY LAYOUT TAB *****/
			
			// Menu_Align	
// props.MenuAlign = getStandardField( widgetData=widgetData, field='Menu_Align');
			
			// Menu_Position         
			props.MenuPosition = getStandardField( widgetData=widgetData, field='Menu_Position');
			
			// Columns (STRUCT)
			props.Columns = getDeviceSizeOptions(widgetData=widgetData, field='Columns', defXSS='hidden-xxs', defXS='hidden-xs', defSM='hidden-sm', defMD='col-md-3', defLG='col-lg-3');

			// Build the Column classes
			props.colClass = getDeviceSizeClassList(props.Columns);
						
			// Special Processing
			if ( useScrollSpy  )
			{
				// Since we know the properties for the FIRST COLUMN (props.colClass) lets calculate the SECOND COLUMN (bodyContentClass)
				columnsClassListArr = Application.ADF.widgets.buildBootstrapTwoColumnClassData(firstColumnClassList=props.colClass,secondColumnClassList="",firstColumnPosition=props.MenuPosition);
			
				if ( ArrayLen(columnsClassListArr) EQ 2 )
				{
					props.colClass = columnsClassListArr[1];
					props.bodyContentClass = columnsClassListArr[2];
				}
					
				// Set Failover DEFAULTS for props.colClass
				if ( LEN(TRIM(props.colClass)) EQ 0 )
				{
					props.colClass = "col-md-3 hidden-xs hidden-sm"; // col-md-push-9
					props.bodyContentClass = "col-md-9";  	
				}
			}
			
//WriteDump(var=props,expand=false);
			
			return props;
		</cfscript>
	</cffunction>
	
	
	<!----------------------------------------
	 	 Render Scroll Spy  
	----------------------------------------->
	<cffunction name="renderScrollSpy" access="public" output="Yes" returntype="void">
		<cfargument name="elementInfo" required="Yes" type="struct">  
		<cfargument name="propsStruct" required="Yes" type="struct">

		<cfscript>
			var props = arguments.propsStruct;
			var elementID = request.element.id;
			var useScrollSpy = false;
			var scrollSpyContainerRenderCSS = '';
			var scrollSpyContainerRenderJS = '';
			var titleData = StructNew();
			var titleProps = StructNew();
			var menuAlignClass = '';
			var menuFontClass = '';
			var menuLinkColor = '';
			var menuLinkHoverColor = '';
			var menuLinkActiveColor = '';
			
			// Init the TitleData and Title Props variables to pass to RenderTitle
			titleData.title = "";
			titleProps.title = StructNew();
			
			if (props.EnableMenu is "Yes")
				useScrollSpy = true;
			
			// Build the Title Data and Title Props to pass to RenderTitle
			if ( len(props.TitleText) ) 
			{
				// Menu Title data Title RenderTitle variables
				titleData.linkURL = ""; // No Link URL needed
				titleData.title = props.TitleText;
				
				// Menu Title drops Title (Struct)
				titleProps.title = props.TitleStyle;
			}
			
			if ( StructKeyExists( props.MenuLinksStyle,'align' ) )
				menuAlignClass = props.MenuLinksStyle.align;
				
			if ( StructKeyExists( props.MenuLinksStyle,'fontclasses' ) )
				menuFontClass = props.MenuLinksStyle.fontclasses;
				
			if ( StructKeyExists( props.MenuLinksStyle,'color' ) )
				menuLinkColor = props.MenuLinksStyle.color;
				
			if ( StructKeyExists( props.MenuLinksStyle,'bgcolor' ) )
				menuLinkHoverColor = props.MenuLinksStyle.bgcolor;
				
			if ( StructKeyExists( props.MenuLinksStyle,'borderColor' ) )
				menuLinkActiveColor = props.MenuLinksStyle.borderColor;
					
			// Set elementInfo back to attributes for the container schedule-standard-render.cfm
			attributes.ElementInfo = arguments.elementinfo;
		</cfscript>
	
		<cfoutput><div id="csw-Scroll-Spy-Container-#elementID#" class="row csw-Scroll-Spy-Container #props.themeClass#"></cfoutput>
	
			<cfif useScrollSpy EQ 1>
				<cfoutput>
					<div id="csw-Scroll-Spy-SideBar-#elementID#" class="#props.colClass# cswSideBar">
						<cfif LEN(TRIM(titleData.title))>
						#renderTitle(titleData,titleProps)#
						</cfif>
						
						<nav class="cswNav csw-scrollspy-container-sidebar csw-scrollspy-dynamic #menuAlignClass# #menuFontClass# cswLinkColor cswLinkHoverColor hidden-print hidden-xs hidden-sm" 
							data-spy="affix" 
							data-offset-top="#props.AffixOffset#" 
							data-csw-scrollspy-fixedtop="#props.FixedMenuTop#"
							data-csw-scrollspy-offset="#props.ScrollOffset#" 
							data-csw-scrollspy-linkcolor="cswLinkColor"></nav>
					</div>
				</cfoutput>
			</cfif>
			
			<cfoutput>
				<!-- // Body Content -->
				<div id="csw-Scroll-Spy-Body-#elementID#" class="#props.bodyContentClass# cswBodyContent">
					<cfinclude template="/commonspot/controls/schedule/schedule-standard-render.cfm">
				</div>
			</cfoutput>
			
			<cfoutput></div></cfoutput>
			
			<cfsavecontent variable="scrollSpyContainerRenderCSS">
			<cfoutput>
					##csw-Scroll-Spy-Container-#elementID# .cswLinkColor {
						color: #menuLinkColor#;
					}
					##csw-Scroll-Spy-Container-#elementID# .cswLinkHoverColor ul.nav > li > a:hover {
						background-color: #menuLinkHoverColor#;
						color: #menuLinkColor#;
					}
					##csw-Scroll-Spy-Container-#elementID# ul.nav > li.active > a.cswLinkColor, 
					##csw-Scroll-Spy-Container-#elementID# ul.nav > li.active > a.cswLinkColor:hover {
						border-left-color: #menuLinkActiveColor#;
					}
			</cfoutput>
			</cfsavecontent>
			
			<cfsavecontent variable="scrollSpyContainerRenderJS">
			<cfoutput>
					jQuery(document).ready(function() {
						// Init Scroll Spy
						initScrollSpy(#props.ScrollOffset#);
					});
			</cfoutput>
			</cfsavecontent>
			
			<cfscript>
				// Load custom javascript and style blocks	
				Server.CommonSpot.udf.resources.addHeaderCSS(scrollSpyContainerRenderCSS, 'TERTIARY');
				Server.CommonSpot.udf.resources.addFooterJS(scrollSpyContainerRenderJS, 'TERTIARY');
			</cfscript>
	</cffunction>
</cfcomponent>
 